@@ -63,7 +63,11 @@ namespace GalleryRemote {
 				return parents;
 			}
 		}
-		
+	
+		public Gallery Gallery {
+			get { return gallery; }
+		}
+
 		public Album (Gallery gallery, string name, int ref_num) 
 		{
 			Name = name;
@@ -82,12 +86,12 @@ namespace GalleryRemote {
 			Add (item, item.DefaultVersionUri.LocalPath);
 		}
 
-		public void Add (FSpot.IBrowsableItem item, string path)		
+		public int Add (FSpot.IBrowsableItem item, string path)		
 		{
 			if (item == null)
 				Console.WriteLine ("NO PHOTO");
 			
-			gallery.AddItem (this, 
+			return gallery.AddItem (this, 
 					 path,
 					 Path.GetFileName (item.DefaultVersionUri.LocalPath),
 					 item.Description, 
@@ -247,6 +251,16 @@ namespace GalleryRemote {
 			}
 		}
 
+		string auth_token;
+		public string AuthToken {
+			get {
+				return auth_token;
+			}
+			set {
+				auth_token = value;
+			}
+		}
+
 		protected GalleryVersion version;
 		
 		public GalleryVersion Version {
@@ -269,7 +283,7 @@ namespace GalleryRemote {
 		public abstract ArrayList FetchAlbums ();
 		public abstract ArrayList FetchAlbumsPrune ();
 		public abstract bool MoveAlbum (Album album, string end_name);
-		public abstract bool AddItem (Album album, string path, string filename, string caption, bool autorotate);
+		public abstract int AddItem (Album album, string path, string filename, string caption, bool autorotate);
 		//public abstract Album AlbumProperties (string album);
 		public abstract bool NewAlbum (string parent_name, string name, string title, string description);
 		public abstract ArrayList FetchAlbumImages (Album album, bool include_ablums);
@@ -407,6 +421,8 @@ namespace GalleryRemote {
 						Console.WriteLine ("StatusText : {0}", data[1]);
 					} else if (data[0].StartsWith ("server_version")) {
 						//FIXME we should use the to determine what capabilities the server has
+					} else if (data[0].StartsWith ("auth_token")) {
+						AuthToken = data[1];
 					} else {
 						Console.WriteLine ("Unparsed Line in ParseLogin(): {0}={1}", data[0], data[1]);
 					}
@@ -495,6 +511,8 @@ namespace GalleryRemote {
 					} else if (data[0].StartsWith ("album_count")) {
 						if (Albums.Count != int.Parse (data[1]))
 							Console.WriteLine ("Parsed album count does not match album_count.  Something is amiss");
+					} else if (data[0].StartsWith ("auth_token")) {
+						AuthToken = data [1];
 					} else {
 						Console.WriteLine ("Unparsed Line in ParseFetchAlbums(): {0}={1}", data[0], data[1]);
 					}
@@ -515,9 +533,43 @@ namespace GalleryRemote {
 			}
 		}
 
-		public bool ParseAddItem (HttpWebResponse response)
+		public int ParseAddItem (HttpWebResponse response)
 		{
-			return ParseBasic (response);
+			string [] data;
+			StreamReader reader = null;
+			ResultCode status = ResultCode.UnknownResponse;
+			string status_text = "Error: Unable to parse server response";
+			int item_id = 0;
+			try {
+
+				reader = findResponse (response);
+				while ((data = GetNextLine (reader)) != null) {
+					if (data[0] == "status") {
+						status = (ResultCode) int.Parse (data [1]);
+					} else if (data[0].StartsWith ("status_text")) {
+						status_text = data[1];
+						Console.WriteLine ("StatusText : {0}", data[1]);
+					} else if (data[0].StartsWith ("auth_token")) {
+						AuthToken = data[1];
+					} else if (data[0].StartsWith ("item_name")) {
+						item_id = int.Parse (data [1]);
+					} else {
+						Console.WriteLine ("Unparsed Line in ParseAddItem(): {0}={1}", data[0], data[1]);
+					}
+				}
+				//Console.WriteLine ("Found: {0} cookies", response.Cookies.Count);
+				if (status != ResultCode.Success) {
+					Console.WriteLine (status_text);
+					throw new GalleryCommandException (status_text, status);
+				}
+
+				return item_id;
+			} finally {
+				if (reader != null)
+					reader.Close ();
+				
+				response.Close ();
+			}
 		}
 
 		public bool ParseNewAlbum (HttpWebResponse response)
@@ -582,6 +634,8 @@ namespace GalleryRemote {
 					} else if (data[0].StartsWith ("status_text")) {
 						status_text = data[1];
 						Console.WriteLine ("StatusText : {0}", data[1]);
+					} else if (data[0].StartsWith ("auth_token")) {
+						AuthToken = data[1];
 					} else {
 						Console.WriteLine ("Unparsed Line in ParseBasic(): {0}={1}", data[0], data[1]);
 					}
@@ -688,7 +742,7 @@ namespace GalleryRemote {
 			return ParseMoveAlbum (client.Submit (uri));
 		}
 		
-		public override bool AddItem (Album album,
+		public override int AddItem (Album album,
 				     string path, 
 				     string filename,
 				     string caption, 
@@ -754,7 +808,6 @@ namespace GalleryRemote {
 			client.Add ("cmd", "fetch-albums-prune");
 			client.Add ("protocol_version", "2.3");
 			client.Add ("check_writable", "no");
-
 			ArrayList a = ParseFetchAlbums (client.Submit (uri));
 			a.Sort();
 			return a;
@@ -872,11 +925,11 @@ namespace GalleryRemote {
 
 		public override void Login (string username, string passwd)
 		{
-			//Console.WriteLine ("Gallery2: Attempting to login");
+			Console.WriteLine ("Gallery2: Attempting to login");
 			FormClient client = new FormClient (cookies);
 			
 			client.Add ("g2_form[cmd]", "login");
-			client.Add ("g2_form[protocol_version]", "2.3");
+			client.Add ("g2_form[protocol_version]", "2.10");
 			client.Add ("g2_form[uname]", username);
 			client.Add ("g2_form[password]", passwd);
 			AddG2Specific (client);
@@ -896,7 +949,7 @@ namespace GalleryRemote {
 			FormClient client = new FormClient (cookies);
 			
 			client.Add ("g2_form[cmd]", "move-album");
-			client.Add ("g2_form[protocol_version]", "2.7");
+			client.Add ("g2_form[protocol_version]", "2.10");
 			client.Add ("g2_form[set_albumName]", album.Name);
 			client.Add ("g2_form[set_destalbumName]", end_name);
 			AddG2Specific (client);
@@ -904,7 +957,7 @@ namespace GalleryRemote {
 			return ParseMoveAlbum (client.Submit (uri));
 		}
 		
-		public override bool AddItem (Album album,
+		public override int AddItem (Album album,
 				     string path, 
 				     string filename,
 				     string caption, 
@@ -913,7 +966,7 @@ namespace GalleryRemote {
 			FormClient client = new FormClient (cookies);
 			
 			client.Add ("g2_form[cmd]", "add-item");
-			client.Add ("g2_form[protocol_version]", "2.9");
+			client.Add ("g2_form[protocol_version]", "2.10");
 			client.Add ("g2_form[set_albumName]", album.Name);
 			client.Add ("g2_form[caption]", caption);
 			client.Add ("g2_form[userfile_name]", filename);
@@ -944,7 +997,7 @@ namespace GalleryRemote {
 		{
 			FormClient client = new FormClient (cookies);
 			client.Add ("g2_form[cmd]", "new-album");
-			client.Add ("g2_form[protocol_version]", "2.8");
+			client.Add ("g2_form[protocol_version]", "2.10");
 			client.Add ("g2_form[set_albumName]", parent_name);
 			client.Add ("g2_form[newAlbumName]", name);
 			client.Add ("g2_form[newAlbumTitle]", title);
@@ -958,7 +1011,7 @@ namespace GalleryRemote {
 		{
 			FormClient client = new FormClient (cookies);
 			client.Add ("g2_form[cmd]", "fetch-album-images");
-			client.Add ("g2_form[protocol_version]","2.3");
+			client.Add ("g2_form[protocol_version]","2.10");
 			client.Add ("g2_form[set_albumName]", album.Name);
 			client.Add ("g2_form[albums_too]", include_ablums ? "yes" : "no");
 			AddG2Specific (client);
@@ -971,7 +1024,7 @@ namespace GalleryRemote {
 		{
 			FormClient client = new FormClient (cookies);
 			client.Add ("g2_form[cmd]", "fetch-albums-prune");
-			client.Add ("g2_form[protocol_version]", "2.3");
+			client.Add ("g2_form[protocol_version]", "2.10");
 			client.Add ("g2_form[check_writable]", "no");
 			AddG2Specific (client);
 			
@@ -982,6 +1035,8 @@ namespace GalleryRemote {
 
 		private void AddG2Specific (FormClient client)
 		{
+			if (AuthToken != null && AuthToken != String.Empty)
+				client.Add("g2_authToken", AuthToken);
 			client.Add("g2_controller", "remote.GalleryRemote");
 		}
 
