@@ -14,6 +14,7 @@ using System.Web;
 using System.Web.Mail;
 
 using FSpot;
+using FSpot.Widgets;
 using LibGPhoto2;
 
 public class MainWindow {
@@ -108,7 +109,8 @@ public class MainWindow {
 	[Glade.Widget] Gtk.Image far_image;
 
 	[Glade.Widget] Gtk.HBox tagbar;
-	[Glade.Widget] Gtk.Entry tag_entry;
+	[Glade.Widget] Gtk.VBox tag_entry_container;
+	TagEntry tag_entry;
 
 	Gtk.Toolbar toolbar;
 
@@ -265,7 +267,6 @@ public class MainWindow {
 		
 		info_box = new InfoBox ();
 		info_box.VersionIdChanged += HandleInfoBoxVersionIdChange;
-        info_box.HeightRequest = 165;
 		left_vbox.PackStart (info_box, false, true, 0);
 		
 		try {
@@ -346,6 +347,7 @@ public class MainWindow {
 
 		photo_view = new PhotoView (query);
 		photo_box.Add (photo_view);
+
 		photo_view.ButtonPressEvent += HandlePhotoViewButtonPressEvent;
 		photo_view.KeyPressEvent += HandlePhotoViewKeyPressEvent;
 		photo_view.UpdateStarted += HandlePhotoViewUpdateStarted;
@@ -356,9 +358,12 @@ public class MainWindow {
 		// Tag typing: focus the tag entry if the user starts typing a tag
 		icon_view.KeyPressEvent += HandlePossibleTagTyping;
 		photo_view.KeyPressEvent += HandlePossibleTagTyping;
+		tag_entry = new TagEntry (db.Tags);
 		tag_entry.KeyPressEvent += HandleTagEntryKeyPressEvent;
-		tag_entry.FocusOutEvent += HandleTagEntryFocusOutEvent;
-		tag_entry.Changed += HandleTagEntryChanged;
+		tag_entry.TagsAttached += HandleTagEntryTagsAttached;
+		tag_entry.TagsRemoved += HandleTagEntryRemoveTags;
+		tag_entry.Activated += HandleTagEntryActivate;
+		tag_entry_container.Add (tag_entry);
 
 		Gtk.Drag.DestSet (photo_view, DestDefaults.All, tag_target_table, 
 				  DragAction.Copy | DragAction.Move); 
@@ -384,7 +389,7 @@ public class MainWindow {
 		tagbar.Hide ();
 		find_bar.Hide ();
 
-        UpdateFindByTagMenu ();
+		UpdateFindByTagMenu ();
 
 		LoadPreference (Preferences.SHOW_TOOLBAR);
 		LoadPreference (Preferences.SHOW_SIDEBAR);
@@ -401,6 +406,7 @@ public class MainWindow {
 		icon_view.SizeAllocated += HandleIconViewReady;
 
 		UpdateToolbar ();
+
 	}
 
 	private Photo CurrentPhoto {
@@ -440,6 +446,8 @@ public class MainWindow {
 				view_notebook.CurrentPage = 0;
 				
 			ColorDialog.Close ();
+			if (photo_view.View.Loupe != null)
+				photo_view.View.Loupe.Destroy ();
 			JumpTo (photo_view.Item.Index);
 			zoom_scale.Value = icon_view.Zoom;
 			break;
@@ -583,6 +591,11 @@ public class MainWindow {
 			if (Changed != null)
 				Changed (this);
 		}
+
+		public void MarkChanged (int index)
+		{
+			throw new System.NotImplementedException ("I didn't think you'd find me");
+		}
 		
 		public IBrowsableItem this [int index] {
 			get {
@@ -639,6 +652,8 @@ public class MainWindow {
 		{
 			if (win.view_mode == ModeType.IconView && Changed != null)
 				Changed (this);
+
+
 		}
 
 		private void HandleSelectionItemsChanged (IBrowsableCollection collection,  BrowsableArgs args)
@@ -659,7 +674,7 @@ public class MainWindow {
 
 		UpdateMenus ();
 		UpdateTagEntryFromSelection ();
-		UpdateStatusLabel();		
+		UpdateStatusLabel();	
 	}
 
 	private void HandleSelectionItemsChanged (IBrowsableCollection collection, BrowsableArgs args)
@@ -1408,7 +1423,8 @@ public class MainWindow {
 		}
 
 		info_display = new FSpot.InfoDisplay ();
-		info_display_window = new Gtk.Dialog ("Metadata Browser", main_window, 
+		info_display_window = new Gtk.Dialog (Catalog.GetString ("Metadata Browser"), 
+						      main_window, 
 						      Gtk.DialogFlags.NoSeparator | Gtk.DialogFlags.DestroyWithParent);
 		info_display_window.SetDefaultSize (400, 400);
 		Gtk.ScrolledWindow scroll = new ScrolledWindow ();
@@ -1462,9 +1478,19 @@ public class MainWindow {
 
 	void HandleExportToFlickr (object sender, EventArgs args)
 	{
-		new FSpot.FlickrExport (new FSpot.PhotoArray (SelectedPhotos ()), true);
+		new FSpot.FlickrExport (FlickrNet.SupportedService.Flickr, new FSpot.PhotoArray (SelectedPhotos ()), true);
 	}
 	
+	void HandleExportToSmugMug (object sender, System.EventArgs args)
+	{
+		new FSpot.SmugMugExport (new PhotoArray (SelectedPhotos ())); 	
+	}
+
+	void HandleExportTo23hq (object sender, EventArgs args)
+	{
+		new FSpot.FlickrExport (FlickrNet.SupportedService.TwentyThreeHQ, new FSpot.PhotoArray (SelectedPhotos ()), true);
+	}
+
 	void HandleExportToFotki (object sender, EventArgs args)
 	{
 		
@@ -1475,8 +1501,7 @@ public class MainWindow {
 		new FSpot.CDExport (new FSpot.PhotoArray (SelectedPhotos ()));
 	}
 
-#if false
-	void TestDisplay ()
+	private void TestDisplay ()
 	{
 		Gtk.Window win = new Gtk.Window ("hello");
 		VBox box = new VBox ();
@@ -1484,7 +1509,6 @@ public class MainWindow {
 		win.Add (box);
 		win.ShowAll ();
 	}
-#endif
 
 	void HandleSendMailCommand (object sender, EventArgs args)
 	{
@@ -1494,10 +1518,11 @@ public class MainWindow {
 
 	public static void HandleHelp (object sender, EventArgs args)
 	{
-		Gnome.Vfs.Url.Show("ghelp:" + 
-				FSpot.Global.HelpDirectory +
-				System.IO.Path.DirectorySeparatorChar +
-				"index.xml");
+		GnomeUtil.ShowHelp ("f-spot.xml", null, Toplevel.Window.Screen, null);
+//		Gnome.Vfs.Url.Show("ghelp:" + 
+//				FSpot.Global.HelpDirectory +
+//				System.IO.Path.DirectorySeparatorChar +
+//				"f-spot.xml");
 	}
 
 	public static void HandleAbout (object sender, EventArgs args)
@@ -1547,7 +1572,7 @@ public class MainWindow {
 
                 new About ("F-Spot", 
 			   FSpot.Defines.VERSION, 
-			   Catalog.GetString ("Copyright \x00a9 2003-2006 Novell Inc."),
+			   Catalog.GetString ("Copyright \x00a9 2003-2007 Novell Inc."),
                            null, authors, new string [0], translators, 
 			   PixbufUtils.LoadFromAssembly("f-spot-logo-tango.svg")).Show();
 	}
@@ -1598,7 +1623,7 @@ public class MainWindow {
 			return;
 		
 		group_selector.Adaptor.OrderAscending = item.Active;
-		icon_view.QueueDraw ();
+		query.RequestReload ();
 
 		// FIXME this is blah...we need UIManager love here
 		if (item != reverse_order)
@@ -1624,7 +1649,7 @@ public class MainWindow {
 		Close();
 	}
 	
-	void Close ()
+	public void Close ()
 	{
 		int x, y, width, height;
 		main_window.GetPosition (out x, out y);
@@ -2081,13 +2106,20 @@ public class MainWindow {
 		case ModeType.PhotoView:
 			break;
 		case ModeType.IconView:		
-			label_str.Append (String.Format (Catalog.GetPluralString ("{0} Photo", "{0} Photos", query.Count), query.Count));
+			label_str.Append (String.Format (Catalog.GetPluralString ("{0} Photo", 
+										  "{0} Photos", 
+										  query.Count), 
+							 query.Count));
 
 			if (PhotoStore.TotalPhotos != query.Count)
-				label_str.Append (String.Format (Catalog.GetString (" out of {0}"), PhotoStore.TotalPhotos));
+				label_str.Append (String.Format (Catalog.GetString (" out of {0}"), 
+								 PhotoStore.TotalPhotos));
 
 			if ((selection != null) && (selection.Count > 0))
-				label_str.Append (String.Format (Catalog.GetPluralString (" ({0} selected)", " ({0} selected)", selection.Count), selection.Count));
+				label_str.Append (String.Format (Catalog.GetPluralString (" ({0} selected)", 
+											  " ({0} selected)", 
+											  selection.Count), 
+								 selection.Count));
 			break;
 		}
 		
@@ -2353,22 +2385,22 @@ public class MainWindow {
 
 	public void HandleRotate90Command (object sender, EventArgs args)
 	{
-        // Don't steal characters from any text entries
-        if (Window.Focus is Gtk.Entry && Gtk.Global.CurrentEvent is Gdk.EventKey) {
-            Window.Focus.ProcessEvent (Gtk.Global.CurrentEvent);
-            return;
-        }
-
+		// Don't steal characters from any text entries
+		if (Window.Focus is Gtk.Entry && Gtk.Global.CurrentEvent is Gdk.EventKey) {
+			Window.Focus.ProcessEvent (Gtk.Global.CurrentEvent);
+			return;
+		}
+		
 		RotateSelectedPictures (GetToplevel (sender), RotateDirection.Clockwise);
 	}
 
 	public void HandleRotate270Command (object sender, EventArgs args)
 	{
-        // Don't steal characters from any text entries
-        if (Window.Focus is Gtk.Entry && Gtk.Global.CurrentEvent is Gdk.EventKey) {
-            Window.Focus.ProcessEvent (Gtk.Global.CurrentEvent);
-            return;
-        }
+		// Don't steal characters from any text entries
+		if (Window.Focus is Gtk.Entry && Gtk.Global.CurrentEvent is Gdk.EventKey) {
+			Window.Focus.ProcessEvent (Gtk.Global.CurrentEvent);
+			return;
+		}
 
 		RotateSelectedPictures (GetToplevel (sender), RotateDirection.Counterclockwise);
 	}
@@ -2573,14 +2605,14 @@ public class MainWindow {
 		return query_widget.TagRequired (tag);
 	}
 
-    private void HandleQueryLogicChanged (object sender, EventArgs args)
-    {
-        HandleFindAddTagWith (null, null);
-    }
-
+	private void HandleQueryLogicChanged (object sender, EventArgs args)
+	{
+		HandleFindAddTagWith (null, null);
+	}
+	
 	public void HandleIncludeTag (object sender, EventArgs args)
 	{
-        ShowQueryWidget ();
+		ShowQueryWidget ();
 		query_widget.Include (tag_selection_widget.TagHighlight);
 	}
 	
@@ -2588,69 +2620,69 @@ public class MainWindow {
 	{
 		query_widget.UnInclude (tag_selection_widget.TagHighlight);
  	}
-
-    void HandleFindByTag (object sender, EventArgs args)
-    {
-        UpdateFindByTagMenu ();
-    }
-
-    public void UpdateFindByTagMenu ()
-    {
-        if (query_widget.Visible) {
-            query_widget.Close ();
-        } else {
-            ShowQueryWidget ();
-        }
-    }
-
-    void HandleFindAddTagWith (object sender, EventArgs args)
-    {
-        if (find_add_tag_with.Submenu != null)
-           find_add_tag_with.Submenu.Dispose ();
-
-        Gtk.Menu submenu = FSpot.Query.TermMenuItem.GetSubmenu (tag_selection_widget.TagHighlight);
-        if (submenu == null)
-            find_add_tag_with.RemoveSubmenu();
-        else
-            find_add_tag_with.Submenu = submenu;
-
-        find_add_tag_with.Sensitive = (submenu != null);
-    }
-
-    public void HandleAddTagToTerm (object sender, EventArgs args)
-    {
-        MenuItem item = sender as MenuItem;
-
-        int item_pos = 0;
-        foreach (MenuItem i in (item.Parent as Menu).Children) {
-            if (item == i) {
-                break;
-            }
-
-            item_pos++;
-        }
-        // account for All and separator menu items
-        item_pos -= 2;
-
-        FSpot.Query.LogicTerm parent_term = (FSpot.Query.LogicTerm) FSpot.Query.LogicWidget.Root.SubTerms [item_pos];
-
-        if (FSpot.Query.LogicWidget.Box != null) {
-            FSpot.Query.Literal after = parent_term.Last as FSpot.Query.Literal;
-            FSpot.Query.LogicWidget.Box.InsertTerm (tag_selection_widget.TagHighlight, parent_term, after);
-        }
-    }
-
+	
+	void HandleFindByTag (object sender, EventArgs args)
+	{
+		UpdateFindByTagMenu ();
+	}
+	
+	public void UpdateFindByTagMenu ()
+	{
+		if (query_widget.Visible) {
+			query_widget.Close ();
+		} else {
+			ShowQueryWidget ();
+		}
+	}
+	
+	void HandleFindAddTagWith (object sender, EventArgs args)
+	{
+		if (find_add_tag_with.Submenu != null)
+			find_add_tag_with.Submenu.Dispose ();
+		
+		Gtk.Menu submenu = FSpot.Query.TermMenuItem.GetSubmenu (tag_selection_widget.TagHighlight);
+		if (submenu == null)
+			find_add_tag_with.RemoveSubmenu();
+		else
+			find_add_tag_with.Submenu = submenu;
+		
+		find_add_tag_with.Sensitive = (submenu != null);
+	}
+	
+	public void HandleAddTagToTerm (object sender, EventArgs args)
+	{
+		MenuItem item = sender as MenuItem;
+		
+		int item_pos = 0;
+		foreach (MenuItem i in (item.Parent as Menu).Children) {
+			if (item == i) {
+				break;
+			}
+			
+			item_pos++;
+		}
+		// account for All and separator menu items
+		item_pos -= 2;
+		
+		FSpot.Query.LogicTerm parent_term = (FSpot.Query.LogicTerm) FSpot.Query.LogicWidget.Root.SubTerms [item_pos];
+		
+		if (FSpot.Query.LogicWidget.Box != null) {
+			FSpot.Query.Literal after = parent_term.Last as FSpot.Query.Literal;
+			FSpot.Query.LogicWidget.Box.InsertTerm (tag_selection_widget.TagHighlight, parent_term, after);
+		}
+	}
+	
 	void HandleFindTagIncluded (Tag t)
 	{
-        ShowQueryWidget ();
+		ShowQueryWidget ();
 		query_widget.Include (new Tag [] {t});
  	}
 	
 	void HandleFindTagRequired (Tag t)
 	{
-        ShowQueryWidget ();
+		ShowQueryWidget ();
 		query_widget.Require (new Tag [] {t});
-    }
+	}
 
 	//
 	// Handle Main Menu 
@@ -2723,7 +2755,7 @@ public class MainWindow {
 				rl_button.Sensitive = true;
 
 				string msg = Catalog.GetPluralString ("Rotate selected photo left",
-				    "Rotate selected photos left", selection.Count);
+								      "Rotate selected photos left", selection.Count);
 				SetTip (rl_button, String.Format (msg, selection.Count));
 			}
 		}
@@ -2830,52 +2862,13 @@ public class MainWindow {
 		y += widget.Allocation.Y;
  	}
 
-	// Tag typing
-
-	private ArrayList selected_photos_tagnames;
+	// Tag typing ...
 
 	private void UpdateTagEntryFromSelection ()
 	{
 		if (!tagbar.Visible)
 			return;
-
-		Hashtable taghash = new Hashtable ();
-
-		Photo [] sel = SelectedPhotos ();
-		for (int i = 0; i < sel.Length; i++) {
-			foreach (Tag tag in sel [i].Tags) {
-				int count = 1;
-
-				if (taghash.Contains (tag))
-					count = ((int) taghash [tag]) + 1;
-
-				if (count <= i)
-					taghash.Remove (tag);
-				else 
-					taghash [tag] = count;
-			}
-			
-			if (taghash.Count == 0)
-				break;
-		}
-
-		selected_photos_tagnames = new ArrayList ();
-		foreach (Tag tag in taghash.Keys)
-			if ((int) (taghash [tag]) == sel.Length)
-				selected_photos_tagnames.Add (tag.Name);
-
-		selected_photos_tagnames.Sort ();
-
-		StringBuilder sb = new StringBuilder ();
-		foreach (string tagname in selected_photos_tagnames) {
-			if (sb.Length > 0)
-				sb.Append (", ");
-
-			sb.Append (tagname);
-		}
-
-		tag_entry.Text = sb.ToString ();
-		ClearTagCompletions ();
+		tag_entry.UpdateFromSelection (SelectedPhotos ());
 	}
 
 	public void HandlePossibleTagTyping (object sender, Gtk.KeyPressEventArgs args)
@@ -2909,67 +2902,6 @@ public class MainWindow {
 	// "Activate" means the user pressed the enter key
 	public void HandleTagEntryActivate (object sender, EventArgs args)
 	{
-		string [] tagnames = GetTypedTagNames ();
-		int [] selected_photos = SelectedIds ();
-
-		if (selected_photos == null || tagnames == null)
-			return;
-
-               int sel_start, sel_end;
-               if (tag_entry.GetSelectionBounds (out sel_start, out sel_end) && tag_completion_index != -1) {
-                       tag_entry.InsertText (", ", ref sel_end);
-                       tag_entry.SelectRegion (-1, -1);
-                       tag_entry.Position = sel_end + 2;
-                       ClearTagCompletions ();
-                       return;
-               }
-
-	       // Add any new tags to the selected photos
-	       Category default_category = null;
-	       Tag [] selection = tag_selection_widget.TagHighlight;
-	       if (selection.Length > 0) {
-		       if (selection [0] is Category)
-			       default_category = (Category) selection [0];
-		       else
-			       default_category = selection [0].Category;
-	       }
-
-		db.BeginTransaction ();
-	       for (int i = 0; i < tagnames.Length; i ++) {
-		       if (tagnames [i].Length == 0)
-			       continue;
-
-		       Tag t = db.Tags.GetTagByName (tagnames [i]);
-
-		       if (t == null) {
-			       t = db.Tags.CreateCategory (default_category, tagnames [i]) as Tag;
-			       db.Tags.Commit (t);
-		       }
-
-		       // Correct for capitalization differences
-		       tagnames [i] = t.Name;
-
-		       Tag [] tags = new Tag [1];
-		       tags [0] = t;
-
-		       foreach (int num in selected_photos)
-			       AddTagExtended (num, tags);
-	       }
-		db.CommitTransaction ();
-	       
-	       // Remove any removed tags from the selected photos
-	       foreach (string tagname in selected_photos_tagnames) {
-		       if (! IsTagInList (tagnames, tagname)) {
-				
-			       Tag tag = db.Tags.GetTagByName (tagname);
-
-			       foreach (int num in selected_photos) {
-				       query.Photos [num].RemoveTag (tag);
-				       query.Commit (num);
-			       }
-		       }
-	       }
-
 	       if (view_mode == ModeType.IconView) {
 		       icon_view.GrabFocus ();
 	       } else {
@@ -2978,6 +2910,52 @@ public class MainWindow {
 	       }
 	}
 
+	private void HandleTagEntryTagsAttached (object o, string [] new_tags)
+	{
+		int [] selected_photos = SelectedIds ();
+		if (selected_photos == null || new_tags == null || new_tags.Length == 0)
+			return;
+
+		Category default_category = null;
+		Tag [] selection = tag_selection_widget.TagHighlight;
+		if (selection.Length > 0) {
+			if (selection [0] is Category)
+				default_category = (Category) selection [0];
+			else
+				default_category = selection [0].Category;
+		}
+
+		db.BeginTransaction ();
+		foreach (string tagname in new_tags) {
+			Tag t = db.Tags.GetTagByName (tagname);
+			if (t == null) {
+				t = db.Tags.CreateCategory (default_category, tagname) as Tag;
+				db.Tags.Commit (t);
+			}
+
+			Tag [] tags = new Tag [1];
+			tags [0] = t;
+
+			foreach (int num in selected_photos)
+				AddTagExtended (num, tags);
+		}
+		db.CommitTransaction ();
+	}
+
+	private void HandleTagEntryRemoveTags (object o, Tag [] remove_tags)
+	{
+		int [] selected_photos = SelectedIds ();
+		if (selected_photos == null || remove_tags == null || remove_tags.Length == 0)
+			return;
+
+		foreach (Tag t in remove_tags) {
+			foreach (int num in selected_photos) {
+				query.Photos [num].RemoveTag (t);
+				query.Commit (num);
+			}
+		}
+	}
+
 	private void HideTagbar ()
 	{
 		if (! tagbar.Visible)
@@ -2993,7 +2971,7 @@ public class MainWindow {
 		else
 			photo_view.View.GrabFocus ();
 
-		ClearTagCompletions ();
+		tag_entry.ClearTagCompletions ();
 	}
 
 	public void HandleTagBarCloseButtonPressed (object sender, EventArgs args)
@@ -3008,98 +2986,7 @@ public class MainWindow {
 		if (args.Event.Key == Gdk.Key.Escape) { 
 			HideTagbar ();
 			args.RetVal = true;
-		} else if (args.Event.Key == Gdk.Key.Tab) {
-			DoTagCompletion ();
-			args.RetVal = true;
-		} else
-			ClearTagCompletions ();
-	}
-
-	bool tag_ignore_changes = false;
-	public void HandleTagEntryChanged (object sender, EventArgs args)
-	{
-		if (tag_ignore_changes)
-			return;
-
-		ClearTagCompletions ();
-	}
-
-	int tag_completion_index = -1;
-	Tag [] tag_completions;
-	string tag_completion_typed_so_far;
-	int tag_completion_typed_position;
-	private void DoTagCompletion ()
-	{
-		string completion;
-		
-		if (tag_completion_index != -1) {
-			tag_completion_index = (tag_completion_index + 1) % tag_completions.Length;
-		} else {
-
-			tag_completion_typed_position = tag_entry.Position;
-		    
-			string right_of_cursor = tag_entry.Text.Substring (tag_completion_typed_position);
-			if (right_of_cursor.Length > 1)
-				return;
-
-			int last_comma = tag_entry.Text.LastIndexOf (',');
-			if (last_comma > tag_completion_typed_position)
-				return;
-
-			tag_completion_typed_so_far = tag_entry.Text.Substring (last_comma + 1).TrimStart (new char [] {' '});
-			if (tag_completion_typed_so_far == null || tag_completion_typed_so_far.Length == 0)
-				return;
-
-			tag_completions = db.Tags.GetTagsByNameStart (tag_completion_typed_so_far);
-			if (tag_completions == null)
-				return;
-
-			tag_completion_index = 0;
 		}
-
-		tag_ignore_changes = true;
-		completion = tag_completions [tag_completion_index].Name.Substring (tag_completion_typed_so_far.Length);
-		tag_entry.Text = tag_entry.Text.Substring (0, tag_completion_typed_position) + completion;
-		tag_ignore_changes = false;
-
-		tag_entry.Position = tag_entry.Text.Length;
-		tag_entry.SelectRegion (tag_completion_typed_position, tag_entry.Text.Length);
-	}
-
-	private void ClearTagCompletions ()
-	{
-		tag_completion_index = -1;
-		tag_completions = null;
-	}
-
-
-	public void HandleTagEntryFocusOutEvent (object sender, EventArgs args)
-	{
-		UpdateTagEntryFromSelection ();
-		//		HideTagbar ();
-	}
-
-	private string [] GetTypedTagNames ()
-	{
-		string [] tagnames = tag_entry.Text.Split (new char [] {','});
-
-		ArrayList list = new ArrayList ();
-		for (int i = 0; i < tagnames.Length; i ++) {
-			string s = tagnames [i].Trim ();
-
-			if (s.Length > 0)
-				list.Add (s);
-		}
-
-		return (string []) (list.ToArray (typeof (string)));
-	}
-
-	private bool IsTagInList (string [] tags, string tag)
-	{
-		foreach (string t in tags)
-			if (t == tag)
-				return true;
-		return false;
 	}
 
 	public string [] SelectedMimeTypes ()
