@@ -184,14 +184,14 @@ public class PhotoView : EventBox {
 
 			string desc = Item.Current.Description;
 			if (description_entry.Text != desc) {
-				description_entry.Text = desc == null ? "" : desc;
+				description_entry.Text = desc == null ? String.Empty : desc;
 			}
 		} else {
 			description_entry.Sensitive = false;
-			description_entry.Text = "";
+			description_entry.Text = String.Empty;
 		}
 
-		if (description_entry.Text != "")
+		if (description_entry.Text != String.Empty)
 			tips.SetTip (description_entry, description_entry.Text, description_entry.Text);
 		else
 			tips.SetTip (description_entry, null, null);
@@ -338,7 +338,8 @@ public class PhotoView : EventBox {
 			if (redeye) {
 				Gdk.Rectangle area = new Gdk.Rectangle (x, y, width, height);
 				edited = PixbufUtils.RemoveRedeye (original_pixbuf, 
-								   area);
+								   area,
+								   (int) Preferences.Get (Preferences.EDIT_REDEYE_THRESHOLD));
 			} else { // Crop (I told you it was ugly)
 				edited = new Pixbuf (original_pixbuf.Colorspace, 
 						     original_pixbuf.HasAlpha, original_pixbuf.BitsPerSample,
@@ -447,7 +448,7 @@ public class PhotoView : EventBox {
 		description_delay = new FSpot.Delay (1000, new GLib.IdleHandler (CommitPendingChanges));
 		this.Destroyed += HandleDestroy;
 
-
+		Name = "ImageContainer";
 		Box vbox = new VBox (false, 6);
 		Add (vbox);
 
@@ -523,6 +524,15 @@ public class PhotoView : EventBox {
 		toolbar_hbox.PackStart (sepia_button, false, true, 0);
 		sepia_button.Clicked += HandleSepiaButtonClicked;
 
+		ItemAction straighten = new TiltEditorAction (photo_view);
+		toolbar_hbox.PackStart (straighten.GetToolButton (false), false, true, 0);
+		
+		ItemAction softfocus = new SoftFocusEditorAction (photo_view);
+		toolbar_hbox.PackStart (softfocus.GetToolButton (false), false, true, 0);
+
+		ItemAction autocolor = new AutoColor (photo_view.Item);
+		toolbar_hbox.PackStart (autocolor.GetToolButton (false), false, true, 0);
+
 		/* Spacer Label */
 		toolbar_hbox.PackStart (new Label (""), true, true, 0);
 
