@@ -11,7 +11,7 @@ public class TrayView : IconView {
 	{
 		DisplayDates = false;
 		DisplayTags = false;
-		cell_border_width = 4;
+		cell_border_width = 10;
 		tag_icon_vspacing = 0;
 		tag_icon_size = 0;
 	}
@@ -55,14 +55,14 @@ public class QueryView : IconView {
 	protected override bool OnPopupMenu ()
 	{
 		PhotoPopup popup = new PhotoPopup ();
-		popup.Activate (this);
+		popup.Activate ();
 		return true;
 	}
 
 	protected override void ContextMenu (Gtk.ButtonPressEventArgs args, int cell_num)
 	{
 		PhotoPopup popup = new PhotoPopup ();
-		popup.Activate (this, args.Event);
+		popup.Activate (this.Toplevel, args.Event);
 	}
 }
 
