@@ -2,28 +2,24 @@ using System;
 using Gtk;
 using Gdk;
 
-public class TagView : Gtk.Widget {
+namespace FSpot {
+public class TagView : EventBox {
 	private int thumbnail_size = 20;
-	private Photo photo;
+	private IBrowsableItem photo;
 	private Tag [] tags;
 	private static int TAG_ICON_VSPACING = 5;
+	bool parent_tip;
 
-	private EventBox parent;
-
-	public TagView (EventBox parent)
+	public TagView ()
 	{
-		this.parent = parent;
-
-		SetFlag (WidgetFlags.NoWindow);
+		VisibleWindow = false;
 	}
 
-	protected TagView (IntPtr raw) : base (raw) {}
-
-	public Photo Current {
+	public IBrowsableItem Current {
 		set {
 			photo = value;
 
-			if (photo != null) {
+			if (photo != null && photo.Tags != null) {
 				SetSizeRequest ((thumbnail_size + TAG_ICON_VSPACING) * photo.Tags.Length,
 						thumbnail_size);
 			} else {
@@ -86,8 +82,9 @@ public class TagView : Gtk.Widget {
 						      RgbDither.None, tag_x, tag_y);
 			tag_x += thumbnail_size + TAG_ICON_VSPACING;
 		}
-		MainWindow.SetTip (parent, String.Join (", ", names));
+		MainWindow.SetTip (this, String.Join (", ", names));
 
 		return base.OnExposeEvent (args);
 	}
 }
+}
