@@ -1,4 +1,5 @@
 using System;
+using System.Threading;
 
 namespace FSpot {
 
@@ -34,9 +35,16 @@ namespace FSpot {
 		private Gtk.ProgressBar progress_bar;
 		private Gtk.Label message_label;
 		private Gtk.Button button;
-		private System.Threading.Thread thread;
 
-		public ThreadProgressDialog (System.Threading.Thread thread, int total) {
+		private Gtk.Button retry_button;
+		private Gtk.Button skip_button;
+		private Gtk.ResponseType error_response;
+		private AutoResetEvent error_response_event;
+		private AutoResetEvent error_response_ack_event;
+
+		private Thread thread;
+
+		public ThreadProgressDialog (Thread thread, int total) {
 			/*
 			if (parent_window)
 				this.TransientFor = parent_window;
@@ -55,6 +63,14 @@ namespace FSpot {
 			progress_bar = new Gtk.ProgressBar ();
 			VBox.PackStart (progress_bar, true, true, 6);
 
+			retry_button = new Gtk.Button (Mono.Unix.Catalog.GetString ("Retry"));
+			retry_button.Clicked += new EventHandler (HandleRetryClicked);
+			skip_button = new Gtk.Button (Mono.Unix.Catalog.GetString ("Skip"));
+			skip_button.Clicked += new EventHandler (HandleSkipClicked);
+
+			ActionArea.Add (retry_button);
+			ActionArea.Add (skip_button);
+
 			button_label = Gtk.Stock.Cancel;
 			button = (Gtk.Button) AddButton (button_label, (int)Gtk.ResponseType.Cancel);
 			
@@ -116,30 +132,63 @@ namespace FSpot {
 			}
 		}
 
+		private bool RetrySkipVisible {
+			set { 
+				retry_button.Visible = skip_button.Visible = value;
+			} 
+		}
+
+		public bool PerformRetrySkip ()
+		{
+			error_response = Gtk.ResponseType.None;
+			RetrySkipVisible = true;
+
+			error_response_event = new AutoResetEvent (false);
+			error_response_ack_event = new AutoResetEvent (false);
+			error_response_event.WaitOne ();
+
+			RetrySkipVisible = false;
+
+			return error_response == Gtk.ResponseType.Yes ? true : false;
+		}
+
 		private void HandleResponse (object obj, Gtk.ResponseArgs args) {
 			this.Destroy ();
 		}
 
-		private bool HandleUpdate () 
+		private bool HandleUpdate ()
 		{
 			message_label.Text = message;
 			progress_bar.Text = progress_text;
-			progress_bar.Fraction = (double) fraction;
+			progress_bar.Fraction = System.Math.Min (1.0, System.Math.Max (0.0, fraction));
 			button.Label = button_label;
 
-			return false;
+			return true;
 		}
 
 		private void HandleDestroy (object sender, EventArgs args)
 		{
-			if (thread.IsAlive) 
-				thread.Abort ();
-
 			delay.Stop ();
+			if (thread.IsAlive) {
+				thread.Abort ();
+			}
 		}
-		
+
+		private void HandleRetryClicked (object obj, EventArgs args)
+		{
+			error_response = Gtk.ResponseType.Yes;
+			error_response_event.Set ();
+		}
+
+		private void HandleSkipClicked (object obj, EventArgs args)
+		{
+			error_response = Gtk.ResponseType.No;
+			error_response_event.Set ();
+		}
+
 		public void Start () {
 			ShowAll ();
+			RetrySkipVisible = false;
 			thread.Start ();
 		}
 	}
